#include <stdint.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include "technix.h"

#define MAX(x, y) (((x) > (y)) ? (x) : (y))
#define MIN(x, y) (((x) < (y)) ? (x) : (y))

#define DISPLAY_UART_ID 0
#define CAR_SENSOR_PORT 0
#define CAR_SENSOR_PIN 0
#define RED_LIGHT_PORT 0
#define RED_LIGHT_PIN 1
#define GREEN_LIGHT_PORT 0
#define GREEN_LIGHT_PIN 2
#define SERVO_DAC_ID 0
#define SERVO_DAC_CHANNEL 0

#define STEP_TIME_MILLIS (10)


void DisplayWrite(const char* text);
void DisplayClear();
void UartSendBuffer(uint32_t uart_id, uint8_t* buffer, size_t buffer_length);
void UartReceiveBuffer(uint32_t spi_id, uint8_t* buffer, size_t buffer_length);
void SpiSendBuffer(uint32_t spi_id, uint8_t* buffer, size_t buffer_length);
void SpiReceiveBuffer(uint32_t spi_id, uint8_t* buffer, size_t buffer_length);
void ConsoleWrite(const char* text);

int main(void)
{
	// Set warning light to red on startup.
	tx_gpio_pin_set(RED_LIGHT_PORT, RED_LIGHT_PIN, true);
	tx_gpio_pin_set(GREEN_LIGHT_PORT, GREEN_LIGHT_PIN, false);

	// Display welcome message.
	DisplayClear();
	DisplayWrite("Welcome\r\nRate: $2/hour");

	while (1)
	{

		// Wait for car to arrive.
		while (!tx_gpio_pin_get(CAR_SENSOR_PORT, CAR_SENSOR_PIN))
			tx_cycle();

		// Display instruction message.
		DisplayClear();
		DisplayWrite("Please wait...\r\n");

		// Car is present, open gate.
		for (size_t i = 0; i < 100; i++)
		{
			tx_dac_write(SERVO_DAC_ID, SERVO_DAC_ID, i * 4095 / 99);
			tx_delay_millis(20);
		}

		// Set light to green.
		tx_gpio_pin_set(RED_LIGHT_PORT, RED_LIGHT_PIN, false);
		tx_gpio_pin_set(GREEN_LIGHT_PORT, GREEN_LIGHT_PIN, true);


		// Display instruction message.
		DisplayClear();
		DisplayWrite("Drive through \r\nslowly.");

		// Wait till car is gone.
		while (tx_gpio_pin_get(CAR_SENSOR_PORT, CAR_SENSOR_PIN))
			tx_cycle();

		// Then wait 2 more seconds.
		tx_delay_millis(2000);

		// Set warning light to red again.
		tx_gpio_pin_set(RED_LIGHT_PORT, RED_LIGHT_PIN, true);
		tx_gpio_pin_set(GREEN_LIGHT_PORT, GREEN_LIGHT_PIN, false);

		// Close the gate.
		for (size_t i = 0; i < 100; i++)
		{
			tx_dac_write(SERVO_DAC_ID, SERVO_DAC_ID, (99 - i) * 4095 / 99);
			tx_delay_millis(20);
		}

		// Display welcome message.
		DisplayClear();
		DisplayWrite("Welcome\r\nRate: $2/hour");

	}
}

// Maximum character count: 34 (2 x 16 display + \r\n)
void DisplayWrite(const char* text)
{
	DisplayClear();
	UartSendBuffer(0, text, strlen(text));
}

void DisplayClear()
{
	uint8_t data[2] = { 0xFE, 0x01 };
	UartSendBuffer(0, data, 2);
}


void ConsoleWrite(const char* text)
{
	UartSendBuffer(0, text, strlen(text));
}



void SpiSendBuffer(uint32_t spi_id, uint8_t* buffer, size_t buffer_length)
{
	for (size_t i = 0; i < buffer_length; i++)
	{
		tx_spi_send(spi_id, buffer[i]);
		tx_delay_micros(10);
	}
}

void SpiReceiveBuffer(uint32_t spi_id, uint8_t* buffer, size_t buffer_length)
{
	for (size_t i = 0; i < buffer_length; i++)
	{
		buffer[i] = tx_spi_receive(spi_id);
	}
}

void UartSendBuffer(uint32_t uart_id, uint8_t* buffer, size_t buffer_length)
{
	for (size_t i = 0; i < buffer_length; i++)
	{
		tx_uart_send(uart_id, buffer[i]);
		tx_delay_micros(10); 
	}
}

void UartReceiveBuffer(uint32_t uart_id, uint8_t* buffer, size_t buffer_length)
{
	for (size_t i = 0; i < buffer_length; i++)
	{
		buffer[i] = tx_uart_receive(uart_id);
	}
}


void StartupAnimation()
{
	// Starting up animation.
	DisplayWrite("Starting up");
	tx_delay_millis(500);
	DisplayClear();
	DisplayWrite("Starting up.");
	tx_delay_millis(500);
	DisplayClear();
	DisplayWrite("Starting up..");
	tx_delay_millis(500);
	DisplayClear();
	DisplayWrite("Starting up...");
	tx_delay_millis(1500);
	DisplayClear();
}
