#ifndef FILE_FOO_SEEN
#define FILE_FOO_SEEN

#include <stdint.h>
#include <stdbool.h>

/////////////////////////////
// Macros
/////////////////////////////

#if defined(_MSC_VER)
#define __TX_EXPORT __declspec(dllexport)
#define __TX_IMPORT __declspec(dllimport)
#elif defined(__GNUC__)
#define __TX_EXPORT __attribute__((visibility("default")))
#define __TX_IMPORT
#else
#define __TX_EXPORT
#define __TX_IMPORT
#pragma warning Unknown dynamic link import/export semantics.
#endif


#ifdef _MSC_VER
# define __TX_INLINE __inline
#elif !defined __STDC_VERSION__ || __STDC_VERSION__ < 199901L
# define __TX_INLINE __inline__
#else
# define __TX_INLINE inline
#endif

/////////////////////////////
// Definitions
/////////////////////////////

// Definition of the simulation host function pointer types.
typedef void (*TX_GPIO_PIN_SET_FUNC)(uint32_t port_id, uint32_t pin_id, bool value);
typedef bool (*TX_GPIO_PIN_GET_FUNC)(uint32_t port_id, uint32_t pin_id);
typedef uint32_t(*TX_ADC_READ_FUNC)(uint32_t adc_id, uint32_t channel_id);
typedef void (*TX_DAC_WRITE_FUNC)(uint32_t dac_id, uint32_t channel_id, int32_t value);
typedef uint8_t(*TX_UART_RECEIVE_FUNC)(uint32_t uart_id);
typedef void (*TX_UART_SEND_FUNC)(uint32_t uart_id, uint8_t value);
typedef uint8_t(*TX_SPI_RECEIVE_FUNC)(uint32_t spi_id);
typedef void (*TX_SPI_SEND_FUNC)(uint32_t spi_id, uint8_t value);
typedef void (*TX_CYCLE_FUNC)();

// Provides the time since startup in the given units.
// Note that the simulation time only increments at tx_cycle or tx_delay calls,
// so this function will not increment the simulation time, 
// it only provides the value of the current simulation time.
// Another consequence is that the resolution of the time functions will depend 
// on how often the tx_cycle and tx_delay functions are called.
// The 'quanta' version is the smallest time unit of the simulation 
// and can be different for every simulation.
typedef uint32_t(*TX_TIME_SECONDS_FUNC)();
typedef uint32_t(*TX_TIME_MILLIS_FUNC)();
typedef uint32_t(*TX_TIME_MICROS_FUNC)();
typedef uint32_t(*TX_TIME_NANOS_FUNC)();
typedef uint32_t(*TX_TIME_QUANTA_FUNC)();

// Suspends the execution for the given time period.
// This function will increment the simulation time.
typedef void (*TX_DELAY_SECONDS_FUNC)(uint32_t delay);
typedef void (*TX_DELAY_MILLIS_FUNC)(uint32_t delay);
typedef void (*TX_DELAY_MICROS_FUNC)(uint32_t delay);
typedef void (*TX_DELAY_NANOS_FUNC)(uint32_t delay);
typedef void (*TX_DELAY_QUANTA_FUNC)(uint32_t delay);

// Defines the data structure that contains the handles to the tx_functions.
typedef struct
{
    TX_GPIO_PIN_SET_FUNC tx_gpio_pin_set;
    TX_GPIO_PIN_GET_FUNC tx_gpio_pin_get;
    TX_ADC_READ_FUNC tx_adc_read;
    TX_DAC_WRITE_FUNC tx_dac_write;
    TX_UART_RECEIVE_FUNC tx_uart_receive;
    TX_UART_SEND_FUNC tx_uart_send;
    TX_SPI_RECEIVE_FUNC tx_spi_receive;
    TX_SPI_SEND_FUNC tx_spi_send;
    TX_CYCLE_FUNC tx_cycle;

    TX_TIME_SECONDS_FUNC tx_time_seconds;
    TX_TIME_MILLIS_FUNC tx_time_millis;
    TX_TIME_MICROS_FUNC tx_time_micros;
    TX_TIME_NANOS_FUNC tx_time_nanos;
    TX_TIME_QUANTA_FUNC tx_time_quanta;

    TX_DELAY_SECONDS_FUNC tx_delay_seconds;
    TX_DELAY_MILLIS_FUNC tx_delay_millis;
    TX_DELAY_MICROS_FUNC tx_delay_micros;
    TX_DELAY_NANOS_FUNC tx_delay_nanos;
    TX_DELAY_QUANTA_FUNC tx_delay_quanta;

} SimulationHost;


// Declaration of the main entry point of the DLL.
// This is the entry point that the simulation host will call first.
// In this function the necessary handles are setup such that
// the target application (that imports this header) can communicate with the simulation host.
//void tx_main(SimulationHost host);
__TX_EXPORT void tx_main(TX_GPIO_PIN_SET_FUNC tx_set);

// Declaration of the GPIO api functions.
void tx_gpio_pin_set(uint32_t port_id, uint32_t pin_id, bool value);
bool tx_gpio_pin_get(uint32_t port_id, uint32_t pin_id);

// Declaration of the Analog api functions.
// Reads the converted value of a single channel of the adc with given id.
// Result is uint32_t, since I have not seen more than 32 bit ADCs. Most ADCs need less than 16 bit.
uint32_t tx_adc_read(uint32_t adc_id, uint32_t channel_id);

// Declaration of the DAC api functions.
void tx_dac_write(uint32_t dac_id, uint32_t channel_id, int32_t value);

void tx_cycle();

// Declaration of the UART api functions.
// Receives a single byte over the UART with given id.
uint8_t tx_uart_receive(uint32_t uart_id);
// Transmits a single byte over the UART with the given id.
void tx_uart_send(uint32_t uart_id, uint8_t data);


// Declaration of the SPI api functions.
// Receives a single byte over the SPI with given id.
uint8_t tx_spi_receive(uint32_t spi_id);
// Transmits a single byte over the SPI with the given id.
void tx_spi_send(uint32_t spi_id, uint8_t data);


// Gets the time since device startup, in the given time unit.
uint32_t tx_time_seconds();
uint32_t tx_time_millis();
uint32_t tx_time_micros();
uint32_t tx_time_nanos();
uint32_t tx_time_quanta();

// Sleeps for given period.
void tx_delay_seconds(uint32_t delay);
void tx_delay_millis(uint32_t delay);
void tx_delay_micros(uint32_t delay);
void tx_delay_nanos(uint32_t delay);
void tx_delay_quanta(uint32_t delay);

#endif /* __TECHNIX_H */

